from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt, to_decimal


class ValorIcmsProprio():

    def __init__(self, base_calculo: float, aliq_icms_proprio: float):
        self.base_calculo = float(base_calculo)
        self.aliq_icms_proprio = float(aliq_icms_proprio)

    def calcular_valor_icms_proprio(self):
        return round_abnt(
            (self.aliq_icms_proprio / 100 * self.base_calculo), 2)



from viggocore.common.subsystem import operation
from viggocore.common.subsystem import manager


class Create(operation.Create):

    def post(self):

        natureza_list = ['Compra', 'Venda', 'Transferência', 'Devolução',
                         'Importação', 'Consignação', 'Outra',
                         'Remessa para fins de demonstração',
                         'Remessa para fins de industrialização']

        domain_org_id = self.entity.id
        if domain_org_id is not None:
            for item in natureza_list:
                natureza = {
                    'domain_org_id': domain_org_id,
                    'titulo': item,
                    'descricao': item
                }
                self.manager.api.natureza_operacaos().create(**natureza)


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)



from viggocore.database import db
from viggocore.common.subsystem import entity


class Csticms(entity.Entity, db.Model):

    attributes = ['cstcsosn', 'descricao', 'trabalha_st']
    attributes += entity.Entity.attributes

    cstcsosn = db.Column(db.String(5), nullable=False, unique=True)
    descricao = db.Column(db.String(200), nullable=False)
    trabalha_st = db.Column(db.Boolean, nullable=False, server_default='false')

    def __init__(self, id, cstcsosn, descricao, trabalha_st=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.cstcsosn = cstcsosn
        self.descricao = descricao
        self.trabalha_st = trabalha_st



import flask

from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common import manager

from viggofiscal.subsystem.parametrizacao.cstcofins.resource import Cstcofins


class List(operation.List):

    def do(self, session, **kwargs):

        if flask.has_request_context():
            token_id = flask.request.headers.get('token')
            if token_id is not None:
                self.token = self.manager.api.tokens().get(id=token_id)
                if self.token is not None:
                    self.user = self.manager.api.users().get(
                        id=self.token.user_id)

        empresa = None
        crt_empresa = None
        if self.user is not None:
            empresa = self.manager.api.domain_orgs().get(id=self.user.domain_id)
        if empresa is not None:
            crt_empresa = empresa.crt
        cst_list = ["2", "4", "8", "49"]

        query = session.query(Cstcofins)

        if crt_empresa.value == 1:
            query = query.filter(Cstcofins.cst.in_(cst_list))

        query = self.manager.apply_filters(query, Cstcofins, **kwargs)

        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Cstcofins, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Cstcofins)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.list = List(self)



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class BaseIpi():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)

    def calcular_base_ipi(self) -> float:
        base_ipi = (self.valor_produto + self.valor_frete + self.valor_seguro + 
                    self.despesas_acessorias)
        return round_abnt(base_ipi, 2)



from enum import Enum
import sqlalchemy
from sqlalchemy import orm, ForeignKeyConstraint

from viggocore.database import db
from viggocore.common.subsystem import entity


class NcmIbpt(entity.Entity, db.Model):

    attributes = ['ncm', 'uf', 'chave', 'versao', 'descricao', 'extipi',
                  'aliq_nacional', 'aliq_importacao', 'aliq_estadual',
                  'aliq_municipal', 'inicio_vigencia', 'fim_vigencia']
    attributes += entity.Entity.attributes

    ncm = db.Column(db.Numeric(8), nullable=False)
    uf = db.Column(db.CHAR(2), nullable=False)
    chave = db.Column(db.String(100), nullable=False)
    versao = db.Column(db.String(100), nullable=False)
    descricao = db.Column(db.String(255), nullable=False)
    aliq_nacional = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_importacao = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_estadual = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_municipal = db.Column(db.Numeric(5, 2), nullable=False)
    inicio_vigencia = db.Column(db.Date(), nullable=False)
    fim_vigencia = db.Column(db.Date(), nullable=False)
    extipi = db.Column(db.Numeric(), nullable=True)

    def __init__(self, id, ncm, uf, chave, versao, descricao, 
                 aliq_nacional, aliq_importacao, aliq_estadual,
                 aliq_municipal, inicio_vigencia, fim_vigencia,
                 extipi=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.ncm = ncm
        self.uf = uf
        self.chave = chave
        self.versao = versao
        self.descricao = descricao
        self.aliq_nacional = aliq_nacional
        self.aliq_importacao = aliq_importacao
        self.aliq_estadual = aliq_estadual
        self.aliq_municipal = aliq_municipal
        self.inicio_vigencia = inicio_vigencia
        self.fim_vigencia = fim_vigencia
        self.extipi = extipi



from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.icms00 \
    import Icms00
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class Icms20():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_ipi: float, valor_desconto: float,
                 aliq_icms_proprio: float, percentual_reducao: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_ipi = float(valor_ipi)
        self.valor_desconto = float(valor_desconto)
        self.aliq_icms_proprio = float(aliq_icms_proprio)
        self.percentual_reducao = float(percentual_reducao)
        self.base_reduzida_icms = BaseReduzidaIcmsProprio(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias,
            valor_desconto, percentual_reducao, valor_ipi)

    def calcular_bc_icms(self) -> float:
        return self.base_reduzida_icms.calcular_bc_icms()

    def base_reduzida_icms_proprio(self):
        return self.base_reduzida_icms.calcular_base_reduzida_icms_proprio()

    def valor_icms_proprio(self) -> float:
        base_reduzida_icms = self.base_reduzida_icms_proprio()
        valor_icms = base_reduzida_icms * (self.aliq_icms_proprio / 100)
        return round_abnt(valor_icms, 2)

    def valor_icms_desonerado(self) -> float:
        icms00 = Icms00(
            self.valor_produto,
            self.valor_frete, self.valor_seguro, self.despesas_acessorias,
            0, self.valor_desconto, self.aliq_icms_proprio)

        valor_icms_normal = icms00.valor_icms_proprio()
        valor_icms_desonerado = valor_icms_normal - self.valor_icms_proprio()
        return round_abnt(valor_icms_desonerado, 2)


from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.icms00 \
    import Icms00
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_proprio \
    import ValorIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt, to_decimal


class Icms51():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_ipi: float, valor_desconto: float,
                 aliq_icms_proprio: float, percentual_reducao: float,
                 percentual_diferimento: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_ipi = float(valor_ipi)
        self.valor_desconto = float(valor_desconto)
        self.aliq_icms_proprio = float(aliq_icms_proprio)
        self.percentual_reducao = float(percentual_reducao)
        self.percentual_diferimento = float(percentual_diferimento)

    def base_icms_proprio(self):
        if self.percentual_reducao == 0:
            self.bc_icms_proprio = BaseIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto, self.valor_ipi)
            return self.bc_icms_proprio.calcular_base_icms_proprio()
        else:
            self.bc_reduzida_icms_proprio = BaseReduzidaIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto,
                self.percentual_reducao, self.valor_ipi)
            return self.bc_reduzida_icms_proprio.\
                calcular_base_reduzida_icms_proprio()

    def valor_icms_operacao(self) -> float:
        return ValorIcmsProprio(
            self.base_icms_proprio(), self.aliq_icms_proprio).\
                calcular_valor_icms_proprio()

    def valor_icms_diferido(self) -> float:
        valor_icms_operacao = self.valor_icms_operacao()
        valor_icms_diferido = (
            valor_icms_operacao * (self.percentual_diferimento / 100))
        return round_abnt(valor_icms_diferido, 2)

    def valor_icms_proprio(self) -> float:
        valor_icms_proprio = (
            self.valor_icms_operacao() - self.valor_icms_diferido())
        return round_abnt(valor_icms_proprio, 2)



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class BaseIcmsST():

    def __init__(self, base_icms_proprio: float, mva: float,
                 valor_ipi: float=0.0):
        self.base_icms_proprio = float(base_icms_proprio)
        self.mva = float(mva)
        self.valor_ipi = float(valor_ipi)

    def calcular_base_icms_st(self) -> float:
        base_icms_st = (
            (self.base_icms_proprio + self.valor_ipi) * (1 + (self.mva / 100)))
        return round_abnt(base_icms_st, 2)



from enum import Enum
import sqlalchemy
from sqlalchemy import orm, ForeignKeyConstraint

from viggocore.database import db
from viggocore.common.subsystem import entity


class Crt(Enum):
    SIMPLES_NACIONAL = 1
    SN_EXC_REC_BRUTA = 2
    REGIME_NORMAL = 3
    MEI = 4


class RegPisCofins(Enum):
    CUMULATIVO = 1
    NAO_CUMULATIVO = 2


class DomainOrg(entity.Entity, db.Model):

    attributes = ['crt', 'cpf_cnpj', 'insc_est', 'razao_social',
                  'nome_fantasia', 'insc_mun', 'cnae', 'cpf_cnpj_contador',
                  'reg_pis_cofins', 'aliq_pis', 'aliq_cofins', 'cred_sn']
    attributes += entity.Entity.attributes

    domain = orm.relationship(
        'Domain', backref=orm.backref('domain_org_domain'))

    crt = db.Column(sqlalchemy.Enum(Crt), nullable=False)
    cpf_cnpj = db.Column(db.String(14), nullable=False)
    insc_est = db.Column(db.String(14), nullable=False)
    razao_social = db.Column(db.String(100), nullable=False)
    nome_fantasia = db.Column(db.String(100), nullable=False)
    insc_mun = db.Column(db.String(15), nullable=True)
    cnae = db.Column(db.String(15), nullable=True)
    cpf_cnpj_contador = db.Column(db.String(14), nullable=True)
    reg_pis_cofins = db.Column(sqlalchemy.Enum(RegPisCofins), nullable=True)
    aliq_pis = db.Column(db.Numeric(7, 2), nullable=True)
    aliq_cofins = db.Column(db.Numeric(7, 2), nullable=True)
    cred_sn = db.Column(db.Numeric(7, 2), nullable=True)

    __table_args__ = (ForeignKeyConstraint(['id'], ['domain.id']),)

    def __init__(self, id, crt, cpf_cnpj, insc_est, razao_social,
                 nome_fantasia, insc_mun=None, cnae=None,
                 cpf_cnpj_contador=None, reg_pis_cofins=None, aliq_pis=None,
                 aliq_cofins=None, cred_sn=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.crt = crt
        self.cpf_cnpj = cpf_cnpj
        self.insc_est = insc_est
        self.razao_social = razao_social
        self.nome_fantasia = nome_fantasia
        self.insc_mun = insc_mun
        self.cnae = cnae
        self.cpf_cnpj_contador = cpf_cnpj_contador
        self.reg_pis_cofins = reg_pis_cofins
        self.aliq_pis = aliq_pis
        self.aliq_cofins = aliq_cofins
        self.cred_sn = cred_sn

    @classmethod
    def individual(cls):
        return 'domain_org'


from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class Ipi50Especifico():

    def __init__(self, base_calculo: float, aliquota_por_unidade: float):
        # A Base de IPI será a Quantidade (qTrib) do produto na operação
        self.base_calculo = float(base_calculo)
        # Valor por Unidade Tributável
        self.aliquota_por_unidade = float(aliquota_por_unidade)

    def valor_ipi(self) -> float:
        valor_ipi = self.aliquota_por_unidade * self.base_calculo
        return round_abnt(valor_ipi, 2)


from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.icms00 \
    import Icms00
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_proprio \
    import ValorIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_st \
    import BaseIcmsST
from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_st \
    import BaseReduzidaIcmsST
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_st \
    import ValorIcmsST


class Icms30():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_ipi: float, valor_desconto: float,
                 aliq_icms_proprio: float, aliq_icms_st: float,
                 mva: float, percentual_reducao_st: float=0.0):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_ipi = float(valor_ipi)
        self.valor_desconto = float(valor_desconto)
        self.aliq_icms_proprio = float(aliq_icms_proprio)
        self.aliq_icms_st = float(aliq_icms_st)
        self.mva = float(mva)
        self.percentual_reducao_st = float(percentual_reducao_st)
        self.bc_icms_proprio = BaseIcmsProprio(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias,
            valor_desconto)

    # ICMS Próprio
    def base_icms_proprio(self) -> float:
        return self.bc_icms_proprio.calcular_base_icms_proprio()

    def valor_icms_proprio(self) -> float:
        valor_icms_proprio = ValorIcmsProprio(
            self.base_icms_proprio(), self.aliq_icms_proprio).\
            calcular_valor_icms_proprio()
        return valor_icms_proprio

    def valor_icms_desonerado(self) -> float:
        return self.valor_icms_proprio()

    # ICMS ST
    def base_icms_st(self) -> float:
        if self.percentual_reducao_st == 0.0:
            self.bc_icms_st = BaseIcmsST(self.base_icms_proprio(), self.mva,
                                         self.valor_ipi)
            return self.bc_icms_st.calcular_base_icms_st()
        else:
            self.bc_reduzida_icms_st = BaseReduzidaIcmsST(
                self.base_icms_proprio(), self.mva, self.percentual_reducao_st,
                self.valor_ipi)
            return self.bc_reduzida_icms_st.calcular_base_reduzida_icms_st()
    
    def base_icms_st_normal(self) -> float:
        self.bc_icms_st = BaseIcmsST(self.base_icms_proprio(), self.mva,
                                     0.0)
        return self.bc_icms_st.calcular_base_icms_st()

    def valor_icms_st(self) -> float:
        valor_icms_st = ValorIcmsST(
            self.base_icms_st(), self.aliq_icms_st, self.valor_icms_proprio()).\
            calcular_valor_icms_st()
        return valor_icms_st